#!/usr/bin/env python
# encoding: utf-8

import ast
from .client import *
from pkg_resources import resource_string
from functools import wraps
from imp import load_source
from logging.handlers import TimedRotatingFileHandler
from tempfile import gettempdir
from copy import deepcopy
import logging as lg
import os
import os.path as osp
import jsonschema as js
import sys
import json

_logger = lg.getLogger(__name__)

class WebHDFSConfig(object):

  default_path = osp.expanduser('~/.webhdfs.cfg')

  def __init__(self, path=None):
      self.path = path or os.getenv('WEBHDFS_CONFIG', self.default_path)
      if osp.exists(self.path):
        try:
          self.config = json.loads(open(self.path).read())
          self.schema = json.loads(resource_string(__name__, 'resources/config_schema.json'))
          #self.schema = open("resources/schema.config").read()
          try:
            js.validate(self.config, self.schema)
          except js.ValidationError as e:
            print e.message
          except js.SchemaError as e:
            print e

        except ParsingError:
          raise HdfsError('Invalid configuration file %r.', self.path)

        _logger.info('Instantiated configuration from %r.', self.path)
      else:
        raise HdfsError('Invalid configuration file %r.', self.path)


  def get_log_handler(self):
      """Configure and return file logging handler."""
      path = osp.join(gettempdir(), 'pywebhdfs.log')
      level = lg.DEBUG
      
      if 'configuration' in self.config:
        configuration = self.config['configuration']
        if 'logging' in configuration:
          logging_config = configuration['logging']
          if 'disable' in logging_config and logging_config['disable'] == True:
            return NullHandler()
          if 'path' in logging_config:
            path = logging_config['path'] # Override default path.
          if 'level' in logging_config:
            level = getattr(lg, logging_config['level'].upper())

      log_handler = TimedRotatingFileHandler(
          path,
          when='midnight', # Daily backups.
          backupCount=5,
          encoding='utf-8',
      )
      fmt = '%(asctime)s\t%(name)-16s\t%(levelname)-5s\t%(message)s'
      log_handler.setFormatter(lg.Formatter(fmt))
      log_handler.setLevel(level)
      return log_handler

  def get_client(self, cluster_name, **kwargs):
      """Load WebHDFS client.

      :param cluster_name: The client to look up. If the cluster name does not
        exist and exception will be raised.
      :param kwargs: additional arguments can be used to overwrite or add some 
        parameters defined in the configuration file.
      """

      config_copy = deepcopy(self.config)
      for cluster in config_copy['clusters']:
        if cluster['name'] == cluster_name:
            # remove the name parameter from the 
            del cluster['name']
            # set overwrite arguments            
            for extra_option in kwargs:
              cluster[extra_option] = kwargs[extra_option]
            return WebHDFSClient(**cluster)

      # the name does not exist
      raise HdfsError('Cluster %s is not defined in configuration file.' % cluster_name)



import re
from threading import Lock
import logging
import os
import logging as lg

try:
    # Python 3
    from urllib.parse import urlparse
except ImportError:  # pragma: no cover
    # Python 2
    from urlparse import urlparse

_logger = lg.getLogger(__name__)

class HdfsError(Exception):

  """Base error class.
  :param message: Error message.
  :param args: optional Message formatting arguments.
  """

  def __init__(self, message, *args):
    super(HdfsError, self).__init__(message % args if args else message)

class StandbyError(HdfsError):
  """StandBy error class, is a subclass of HdfsError thrown when the namenode is not in active State.

  :param message: Error message.
  :param args: optional Message formatting arguments.

  """
  def __init__(self, message, *args):
    super(StandbyError, self).__init__(message % args if args else message)

class FederationError(HdfsError):
  """Federation error class, is a subclass of HdfsError thrown when not suitable mount point is found.

  :param message: Error message.
  :param args: optional Message formatting arguments.

  """
  def __init__(self, message, *args):
    super(FederationError, self).__init__(message % args if args else message)

class SyncHostsList(object):
  """SyncHostList, is an just the encapsulation of the list of hosts passed to the WebHDFSClient
     this class helps to synchronize the access to the hosts list and provides a way to manage
     failover between namenodes in a HA cluster a thread secure way.

  :param hosts: list of namenodes urls.
  """

  def __init__(self,nameservices):
    self.nameservices = nameservices
    self.lock = Lock()

  def __repr__(self):
    return self.nameservices.__repr__()

  def resolve_hosts_from_path(self,hdfs_path):
    # found out which mount to use for that particular hdfs path
    _logger.debug('Resolve host url for hdfs path %s.', hdfs_path)
    curmount=None
    hosts = None
    for nameservice in self.nameservices:
      for mount in nameservice['mounts']:
        if (hdfs_path.rstrip(os.sep) + os.sep).startswith(mount.rstrip(os.sep) + os.sep):
          if not curmount:
            # first mount
            curmount = mount
            hosts = nameservice['urls']
          else:
            if (mount.rstrip(os.sep) + os.sep).startswith(curmount.rstrip(os.sep) + os.sep):
              # mount is a subdirectory of current mount
              curmount = mount
              hosts =  nameservice['urls']

    if not hosts:
      raise FederationError('Could not resolve any nameservice mount for hdfs path %s".', hdfs_path)

    _logger.debug('Using nameservice %s with mount point %s.', hosts, curmount)
    return hosts

  def get_active_host(self,hdfs_path):
    host=None
    with self.lock:
      hosts=self.resolve_hosts_from_path(hdfs_path)
    return hosts[0]

  def get_host_count(self,hdfs_path):
    return len(self.resolve_hosts_from_path(hdfs_path))

  def switch_active_host(self,url,hdfs_path):
    with self.lock:
      hosts=self.resolve_hosts_from_path(hdfs_path)
      if url != hosts[0]:
        # apparently some other process already switched that namenode 
        return False
      else:
        current_active= hosts.pop(0)
        hosts.append(current_active)
        return True

# This is just to get rid of the annoying requests_kerberos errors logs
class AnnoyingErrorsFilter(logging.Filter):
    def filter(self, record):
      if "requests_kerberos" in record.name and 'Mutual authentication unavailable' in record.msg:
        record.lineno = logging.WARNING
      return True



#!/usr/bin/env python
# encoding: utf-8

"""pywebhdfs: python Web HDFS Client."""

from .client import WebHDFSClient
import logging as lg

__version__ = '1.0.1'
__author__  = 'Yassine Azzouz'
__license__ = 'MIT'
